@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,9 +15,8 @@
 --------------------------------------------------------------------------------
 */
 
+#pragma once
 
-#ifndef POLYMAKE_IDEAL_SINGULAR_INIT_H
-#define POLYMAKE_IDEAL_SINGULAR_INIT_H
 
 // polymake includes
 #include "polymake/client.h"
@@ -45,4 +44,3 @@ namespace singular {
 } // end namespace polymake
 
 
-#endif
