@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_INTERNAL_ARRAY_H
-#define POLYMAKE_INTERNAL_ARRAY_H
+#pragma once
 
 namespace pm {
 
@@ -26,7 +25,6 @@ class Array;
 
 }
 
-#endif // POLYMAKE_INTERNAL_ARRAY_H
 
 // Local Variables:
 // mode:C++
