@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2016
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2023
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -14,12 +15,11 @@
 --------------------------------------------------------------------------------
 */
 
+#pragma once
 /** @file Rational.h
     @brief Implementation of pm::Rational class
 */
 
-#ifndef POLYMAKE_RATIONAL_H
-#define POLYMAKE_RATIONAL_H
 
 #include "polymake/Integer.h"
 
@@ -27,11 +27,11 @@ namespace pm {
 
 /// data from third parties can't have infinite values
 constexpr bool isfinite(const __mpq_struct&) { return true; }
-constexpr int isinf(const __mpq_struct&) { return 0; }
+constexpr Int isinf(const __mpq_struct&) { return 0; }
 
 bool isfinite(const Rational& a) noexcept;
-int isinf(const Rational& a) noexcept;
-int sign(const Rational& a) noexcept;
+Int isinf(const Rational& a) noexcept;
+Int sign(const Rational& a) noexcept;
 
 template <> struct spec_object_traits<Rational>;
 
@@ -251,7 +251,7 @@ public:
       set_data(b[0], 1, initialized::no);
    }
 
-   Rational(long b=0)
+   Rational(long b = 0)
       : Rational(b, 1) {}
 
    Rational(int b)
@@ -259,9 +259,9 @@ public:
 
    /// Construct an infinite value
    static
-   Rational infinity(int sgn)
+   Rational infinity(Int sgn)
    {
-      assert(sgn==1 || sgn==-1);
+      assert(sgn == 1 || sgn == -1);
       Rational result(nullptr);
       set_inf(&result, sgn, initialized::no);
       return result;
@@ -472,6 +472,11 @@ public:
       set_data(num, den, initialized::yes);
       return *this;
    }
+   Rational& set(mpz_t&& num, mpz_t&& den)
+   {
+      set_data(num[0], den[0], initialized::yes);
+      return *this;
+   }
 
    Rational& copy_from(mpz_srcptr num_src, mpz_srcptr den_src)
    {
@@ -543,7 +548,7 @@ public:
       if (__builtin_expect(isfinite(*this), 1))
          return mpq_get_d(this);
       else
-         return isinf(*this) * std::numeric_limits<double>::infinity();
+         return double(isinf(*this)) * std::numeric_limits<double>::infinity();
    }
 
    explicit operator long() const
@@ -553,13 +558,6 @@ public:
       return static_cast<long>(numerator(*this));
    }
 
-   explicit operator int() const
-   {
-      if (!is_integral())
-         throw GMP::BadCast("non-integral number");
-      return static_cast<int>(numerator(*this));
-   }
-
    mpq_srcptr get_rep() const noexcept { return this; }
 
    /// Unary operators
@@ -629,7 +627,7 @@ public:
          else
             set_inf(this, 1, b);
       } else {
-         if (isinf(*this)+isinf(b)==0)
+         if (isinf(*this)+isinf(b) == 0)
             throw GMP::NaN();
       }
       return *this;
@@ -645,7 +643,7 @@ public:
          else
             set_inf(&result, 1, b);
       } else {
-         if (isinf(a)+isinf(b)==0)
+         if (isinf(a)+isinf(b) == 0)
             throw GMP::NaN();
          set_inf(&result, a);
       }
@@ -676,7 +674,7 @@ public:
          else
             set_inf(this, 1, b);
       } else {
-         if (isinf(*this)+isinf(b)==0)
+         if (isinf(*this)+isinf(b) == 0)
             throw GMP::NaN();
       }
       return *this;
@@ -694,7 +692,7 @@ public:
             set_inf(&result, 1, b);
          }
       } else {
-         if (isinf(a)+isinf(b)==0)
+         if (isinf(a)+isinf(b) == 0)
             throw GMP::NaN();
          set_inf(&result, a);
       }
@@ -786,7 +784,7 @@ public:
          else
             set_inf(this, -1, b);
       } else {
-         if (isinf(*this)==isinf(b))
+         if (isinf(*this) == isinf(b))
             throw GMP::NaN();
       }
       return *this;
@@ -802,7 +800,7 @@ public:
          else
             set_inf(&result, -1, b);
       } else {
-         if (isinf(a)==isinf(b))
+         if (isinf(a) == isinf(b))
             throw GMP::NaN();
          set_inf(&result, a);
       }
@@ -833,7 +831,7 @@ public:
          else
             set_inf(this, -1, b);
       } else {
-         if (isinf(*this)==isinf(b))
+         if (isinf(*this) == isinf(b))
             throw GMP::NaN();
       }
       return *this;
@@ -851,7 +849,7 @@ public:
             set_inf(&result, -1, b);
          }
       } else {
-         if (isinf(a)==isinf(b))
+         if (isinf(a) == isinf(b))
             throw GMP::NaN();
          set_inf(&result, a);
       }
@@ -1348,7 +1346,7 @@ public:
    friend
    Rational&& operator>> (Rational&& a, long k)
    {
-      return std::move(a >> k);
+      return std::move(a >>= k);
    }
    friend
    Rational operator>> (const Rational& a, int k)
@@ -1411,7 +1409,7 @@ public:
    /// Comparison. 
    /// The magnitude of the return value is arbitrary, only its sign is relevant.
 
-   int compare(const Rational& b) const
+   Int compare(const Rational& b) const
    {
       if (__builtin_expect(isfinite(*this) && isfinite(b), 1))
          return mpq_cmp(this, &b);
@@ -1419,7 +1417,7 @@ public:
          return isinf(*this)-isinf(b);
    }
 
-   int compare(const Integer& b) const
+   Int compare(const Integer& b) const
    {
       if (__builtin_expect(isfinite(*this) && isfinite(b), 1)) {
          if (__builtin_expect(b.is_zero(), 0))
@@ -1432,7 +1430,7 @@ public:
       }
    }
 
-   int compare(long b) const
+   Int compare(long b) const
    {
       if (__builtin_expect(isfinite(*this), 1)) {
          if (__builtin_expect(!b, 0))
@@ -1445,10 +1443,10 @@ public:
       }
    }
 
-   int compare(int b) const { return compare(long(b)); }
+   Int compare(int b) const { return compare(long(b)); }
 
    /// Comparison. 
-   int compare(double b) const
+   Int compare(double b) const
    {
       if (__builtin_expect(isfinite(*this) && isfinite(b), 1)) {
          return is_integral()
@@ -1467,7 +1465,7 @@ public:
    {
       if (__builtin_expect(isfinite(a) && isfinite(b), 1))
          return mpq_equal(&a, &b);
-      return isinf(a)==isinf(b);
+      return isinf(a) == isinf(b);
    }
 
    friend
@@ -1475,7 +1473,7 @@ public:
    {
       if (__builtin_expect(isfinite(a) && isfinite(b), 1))
          return a.is_integral() && numerator(a)==b;
-      return isinf(a)==isinf(b);
+      return isinf(a) == isinf(b);
    }
 
    friend
@@ -1665,7 +1663,7 @@ public:
       } else {
          throw GMP::ZeroDivide();
       }
-      if (a<0 && k % 2)
+      if (a<0 && k%2)
          result.negate();
       return result;
    }
@@ -1728,7 +1726,7 @@ protected:
    void div_thru_Integer(const Rational& a, const Integer& b);
 
    static
-   void set_inf(mpq_ptr me, int sign, initialized st=initialized::yes)
+   void set_inf(mpq_ptr me, Int sign, initialized st=initialized::yes)
    {
       Integer::set_inf(mpq_numref(me), sign, st);
       Integer::set_finite(mpq_denref(me), 1, st);
@@ -1746,23 +1744,23 @@ protected:
       Integer::set_finite(mpq_denref(me), 1, st);
    }
    static
-   void set_inf(mpq_ptr me, int sign, long inv, initialized st=initialized::yes)
+   void set_inf(mpq_ptr me, Int sign, long inv, initialized st=initialized::yes)
    {
       Integer::set_inf(mpq_numref(me), sign, inv, st);
       Integer::set_finite(mpq_denref(me), 1, st);
    }
    static
-   void set_inf(mpq_ptr me, int sign, const __mpq_struct& inv, initialized st=initialized::yes)
+   void set_inf(mpq_ptr me, Int sign, const __mpq_struct& inv, initialized st=initialized::yes)
    {
       set_inf(me, sign, mpq_sgn(&inv), st);
    }
    static
-   void set_inf(mpq_ptr me, int sign, const __mpz_struct& inv, initialized st=initialized::yes)
+   void set_inf(mpq_ptr me, Int sign, const __mpz_struct& inv, initialized st=initialized::yes)
    {
       set_inf(me, sign, mpz_sgn(&inv), st);
    }
    static
-   void set_inf(mpq_ptr me, int sign, const Integer& inv, initialized st=initialized::yes)
+   void set_inf(mpq_ptr me, Int sign, const Integer& inv, initialized st=initialized::yes)
    {
       set_inf(me, sign, mpz_sgn(&inv), st);
    }
@@ -1925,17 +1923,17 @@ namespace pm {
 inline
 bool isfinite(const Rational& a) noexcept
 {
-   return mpq_numref(a.get_rep())->_mp_alloc;
+   return Integer::isfinite_rep(mpq_numref(a.get_rep()));
 }
 
 inline
-int isinf(const Rational& a) noexcept
+Int isinf(const Rational& a) noexcept
 {
    return isfinite(a) ? 0 : mpq_numref(a.get_rep())->_mp_size;
 }
 
 inline
-int sign(const Rational& a) noexcept
+Int sign(const Rational& a) noexcept
 {
    return mpq_sgn(a.get_rep());
 }
@@ -1990,8 +1988,8 @@ struct cmp_scalar<Rational, Rational, void>
 template <bool is_numerator, typename TPart>
 struct object_traits<RationalParticle<is_numerator, TPart>>
    : object_traits<TPart> {
-   typedef TPart proxy_for;
-   static const bool is_temporary=true, is_persistent=false;
+   using proxy_for = TPart;
+   static constexpr bool is_temporary=true, is_persistent=false;
 };
 
 template <>
@@ -2033,11 +2031,6 @@ public:
    size_t operator() (const Rational& a) const { return __builtin_expect(isfinite(a), 1) ? impl(a.get_rep()) : 0; }
 };
 
-template <>
-struct algebraic_traits<int> {
-   typedef Rational field_type;
-};
-
 template <>
 struct algebraic_traits<long> {
    typedef Rational field_type;
@@ -2082,12 +2075,13 @@ Integer& Integer::operator= (Rational&& b)
    return *this;
 }
 
+Rational pow(const Rational& base, long exp);
+
 }
 namespace polymake {
    using pm::Rational;
 }
 
-#endif // POLYMAKE_RATIONAL_H
 
 // Local Variables:
 // mode:C++
